-- Font Registration
RegisterFontFile('wmk')
fontId = RegisterFontId('Freedom Font')

-- BLIP
local markerNames = {}

function tvRP.addBlip(x, y, z, idtype, idcolor, text)
  if x == nil or y == nil or z == nil or idtype == nil or idcolor == nil then
    return
  end
  
  local blip = AddBlipForCoord(x + 0.1, y + 0.1, z + 0.1)
  SetBlipSprite(blip, idtype)
  SetBlipAsShortRange(blip, true)
  SetBlipColour(blip, idcolor)
  SetBlipScale(blip, 0.7) -- 0.7 is a typical scale, 0.9 is default

  if text then
    BeginTextCommandSetBlipName("STRING")
    AddTextComponentString(text)
    EndTextCommandSetBlipName(blip)
  end

  return blip
end

function carwashdraw(x, y, width, height, scale, text, r, g, b, a, outline, font, center)
  SetTextFont(font or 0) -- Default font if not provided
  SetTextProportional(0)
  SetTextScale(scale, scale)
  SetTextColour(r or 255, g or 255, b or 255, a or 255)
  SetTextDropShadow(0, 0, 0, 0, 255)
  SetTextEdge(1, 0, 0, 0, 255)
  SetTextDropShadow()
  SetTextCentre(center or false)
  if outline then
    SetTextOutline()
  end
  SetTextEntry("STRING")
  AddTextComponentString(text)
  DrawText(x - (width / 2), y - (height / 2) + 0.002)
end

-- Remove blip by native id
function tvRP.removeBlip(id)
  if id then
    RemoveBlip(id)
  end
end

function tvRP.addMarkerNames(theX, theY, theZ, theText, theFont, theScale)
  table.insert(markerNames, {x = theX, y = theY, z = theZ, text = theText, font = theFont, scale = theScale})
end

local named_blips = {}

-- Set a named blip (same as addBlip but for a unique name, add or update)
-- Return native id
function tvRP.setNamedBlip(name, x, y, z, idtype, idcolor, text)
  if not name or x == nil or y == nil or z == nil or idtype == nil or idcolor == nil then
    return
  end
  
  tvRP.removeNamedBlip(name) -- Remove old one
  named_blips[name] = tvRP.addBlip(x, y, z, idtype, idcolor, text)
  return named_blips[name]
end

-- Remove a named blip
function tvRP.removeNamedBlip(name)
  if named_blips[name] then
    tvRP.removeBlip(named_blips[name])
    named_blips[name] = nil
  end
end


-- GPS

-- set the GPS destination marker coordinates
function tvRP.setGPS(x,y)
  SetNewWaypoint(x+0.0001,y+0.0001)
end

-- set route to native blip id
function tvRP.setBlipRoute(id)
  SetBlipRoute(id,true)
end

-- MARKER

local markers = {}
local marker_ids = Tools.newIDGenerator()
local named_markers = {}

-- add a circular marker to the game map
-- return marker id
function tvRP.addMarker(x,y,z,sx,sy,sz,r,g,b,a)
  local marker = {x=x,y=y,z=z,sx=sx,sy=sy,sz=sz,r=r,g=g,b=b,a=a}


  -- default values
  if marker.sx == nil then marker.sx = 2.0 end
  if marker.sy == nil then marker.sy = 2.0 end
  if marker.sz == nil then marker.sz = 0.7 end

  if marker.r == nil then marker.r = 0 end
  if marker.g == nil then marker.g = 155 end
  if marker.b == nil then marker.b = 255 end
  if marker.a == nil then marker.a = 200 end

  -- fix gta5 integer -> double issue
  marker.x = marker.x+0.001
  marker.y = marker.y+0.001
  marker.z = marker.z+0.001
  marker.sx = marker.sx+0.001
  marker.sy = marker.sy+0.001
  marker.sz = marker.sz+0.001

  local id = marker_ids:gen()
  markers[id] = marker

  return id
end

-- remove marker
function tvRP.removeMarker(id)
  if markers[id] ~= nil then
    markers[id] = nil
    marker_ids:free(id)
  end
end

-- set a named marker (same as addMarker but for a unique name, add or update)
-- return id
function tvRP.setNamedMarker(name,x,y,z,sx,sy,sz,r,g,b,a)
  tvRP.removeNamedMarker(name) -- remove old marker

  named_markers[name] = tvRP.addMarker(x,y,z,sx,sy,sz,r,g,b,a)
  return named_markers[name]
end

function tvRP.removeNamedMarker(name)
  if named_markers[name] ~= nil then
    tvRP.removeMarker(named_markers[name])
    named_markers[name] = nil
  end
end

local markerSigns = {}
local markerSign_id = Tools.newIDGenerator()
local named_markerSign = {}
function tvRP.addMarkerSign(Sid,x,y,z,sx,sy,sz,r,g,b,a,spin,bobble,rotate)
  local markerSign = {Sid=Sid,x=x,y=y,z=z,sx=sx,sy=sy,sz=sz,r=r,g=g,b=b,a=a,spin=spin,bobble=bobble,rotate=rotate}


  -- default values
  if markerSign.sx == nil then markerSign.sx = 2.0 end
  if markerSign.sy == nil then markerSign.sy = 2.0 end
  if markerSign.sz == nil then markerSign.sz = 0.7 end

  if markerSign.r == nil then markerSign.r = 0 end
  if markerSign.g == nil then markerSign.g = 155 end
  if markerSign.b == nil then markerSign.b = 255 end
  if markerSign.a == nil then markerSign.a = 200 end

  -- fix gta5 integer -> double issue
  markerSign.x = markerSign.x+0.001
  markerSign.y = markerSign.y+0.001
  markerSign.z = markerSign.z+0.001
  markerSign.sx = markerSign.sx+0.001
  markerSign.sy = markerSign.sy+0.001
  markerSign.sz = markerSign.sz+0.001

  local id = markerSign_id:gen()
  markerSigns[id] = markerSign

  return id
end

function tvRP.removeMarkerSign(id)
  if markerSigns[id] ~= nil then
    markerSigns[id] = nil
    markerSign_id:free(id)
  end
end

function tvRP.setNamedSign(name,Sid,x,y,z,sx,sy,sz,r,g,b,a)
  tvRP.removeNamedSign(name) -- remove old marker

  named_markerSign[name] = tvRP.addMarkerSign(Sid,x,y,z,sx,sy,sz,r,g,b,a)
  return named_markerSign[name]
end

function tvRP.removeNamedSign(name)
  if named_markerSign[name] ~= nil then
    tvRP.removeMarkerSign(named_markerSign[name])
    named_markerSign[name] = nil
  end
end

local function DrawText3D(x,y,z, text, scl, font) 
  local onScreen,_x,_y=World3dToScreen2d(x,y,z)
  local px,py,pz=table.unpack(GetGameplayCamCoords())
  local dist = GetDistanceBetweenCoords(px,py,pz, x,y,z, 1)

  local scale = (1/dist)*scl
  local fov = (1/GetGameplayCamFov())*100
  local scale = scale*fov
  
  if onScreen then
    SetTextScale(0.0*scale, 1.1*scale)
    SetTextFont(font)
    SetTextProportional(1)
    -- SetTextScale(0.0, 0.55)
    SetTextColour(255, 255, 255, 255)
    SetTextDropshadow(0, 0, 0, 0, 255)
    SetTextEdge(2, 0, 0, 0, 150)
    SetTextDropShadow()
    SetTextOutline()
    SetTextEntry("STRING")
    SetTextCentre(1)
    AddTextComponentString(text)
    DrawText(_x,_y)
  end
end

-- markers draw loop
Citizen.CreateThread(function()
  while true do
    for k,v in pairs(markers) do
      while Vdist2(GetEntityCoords(PlayerPedId()), v.x, v.y, v.z) <= 20.0 do
        DrawMarker(1,v.x,v.y,v.z,0,0,0,0,0,0,v.sx,v.sy,v.sz,v.r,v.g,v.b,v.a,0,0,0,0)
        Wait(0)
      end
    end
    Wait(500)
  end
end)
Citizen.CreateThread(function()
  while true do
    for k,v in pairs(markerSigns) do
      while Vdist2(GetEntityCoords(PlayerPedId()), v.x, v.y, v.z) <= 20.0 do
        DrawMarker(v.Sid,v.x,v.y,v.z+1.4,0,0,0,0,0,0,v.sx,v.sy,v.sz,v.r,v.g,v.b,v.a,v.bobble,true,0,v.spin,v.rotate)
        Wait(0)
      end
    end
    Wait(500)
  end
end)
Citizen.CreateThread(function()
  while true do
    for k,v in pairs(markerNames) do
      while Vdist2(GetEntityCoords(PlayerPedId()), v.x, v.y, v.z) <= 5.0 do
        DrawText3D(v.x,v.y,v.z, v.text, v.scale, v.font)
        Wait(0)
      end
    end
    Wait(500)
  end
end)
-- AREA

local areas = {}

-- create/update a cylinder area
function tvRP.setArea(name,x,y,z,radius,height)
  local area = {x=x+0.001,y=y+0.001,z=z+0.001,radius=radius,height=height}

  -- default values
  if area.height == nil then area.height = 6 end

  areas[name] = area
end

Citizen.CreateThread(function()
  local minimap = RequestScaleformMovie("minimap")
  SetRadarBigmapEnabled(true, false)
  Wait(1000)
  SetRadarBigmapEnabled(false, false)
  while true do
      Wait(1000)
      BeginScaleformMovieMethod(minimap, "SETUP_HEALTH_ARMOUR")
      ScaleformMovieMethodAddParamInt(3)
      EndScaleformMovieMethod()
  end
end)

-- remove area
function tvRP.removeArea(name)
  if areas[name] ~= nil then
    areas[name] = nil
  end
end

-- DOOR

-- set the closest door state
-- doordef: .model or .modelhash
-- locked: boolean
-- doorswing: -1 to 1
function tvRP.setStateOfClosestDoor(doordef, locked, doorswing)
  local x,y,z = tvRP.getPosition()
  local hash = doordef.modelhash
  if hash == nil then
    hash = GetHashKey(doordef.model)
  end

  SetStateOfClosestDoorOfType(hash,x,y,z,locked,doorswing+0.0001)
end

function tvRP.openClosestDoor(doordef)
  tvRP.setStateOfClosestDoor(doordef, false, 0)
end

function tvRP.closeClosestDoor(doordef)
  tvRP.setStateOfClosestDoor(doordef, true, 0)
end

local getLocalPed = PlayerPedId
local getECoords = GetEntityCoords
local yieldThread = Citizen.Wait

_GPED = getLocalPed()
_GCOORDS = vector3(0.0,0.0,0.0)
_PLAYERCOORDS = getECoords(_GPED)

local updateGlobalVariables = function()
    while 1 do
        _GPED = getLocalPed()
        _GCOORDS = GetEntityCoords(_GPED)
        _PLAYERCOORDS = getECoords(_GPED)
        yieldThread(430)
    end
end

CreateThread(updateGlobalVariables)


function drawInsideText(text, x, y, scale, r, g, b)
	SetTextFont(4)
	SetTextCentre(1)
	SetTextProportional(0)
	SetTextScale(scale, scale)
	SetTextDropShadow(30, 5, 5, 5, 255)
	SetTextEntry("STRING")
	SetTextColour(r, g, b, 255)
	AddTextComponentString(text)
	DrawText(x, y)
end

local areas = {}


-- create/update a cylinder area
function tvRP.setArea(name,x,y,z,radius,height,cbs)

  if cbs == nil then 
    local area = {name = name,x=x+0.001,y=y+0.001,z=z+0.001,radius=radius,height=height, pos = vector3(x+0.001,y+0.001,z+0.001)}
    -- default values
    if area.height == nil then area.height = 6 end
    areas[name] = area
  else
    local area = {name = name,x=x+0.001,y=y+0.001,z=z+0.001,radius=radius,height=height, pos = vector3(x+0.001,y+0.001,z+0.001), ent = cbs.ent, leave = cbs.leave}
    -- default values
    if area.height == nil then area.height = 6 end
    areas[name] = area
  end
end

-- remove area
function tvRP.removeArea(name)

  if areas[name] ~= nil then
    areaInside = nil
  playerInside = false
    insideName = ''
    areas[name] = nil
  end
end

CreateThread(function()
  while true do 
      Wait(300)
      while playerInside and areaInside ~= nil and insideName ~= '' do 
          Wait(1)

          -- Afișează mesajul cu insideName folosind TextUI
          exports["brx_ui"]:Open("[E] pentru a deschide " .. insideName .. "", "lightblue", "left")

          -- Verifică dacă jucătorul apasă tasta corespunzătoare
          if IsControlJustPressed(1, 51) then 
              vRPserver.enterArea({areaInside})
          end
      end

      -- Închide TextUI dacă jucătorul nu este în interiorul zonei
      exports["brx_ui"]:Close()
  end
end)




  
  local player_in 
  local v 
  CreateThread(function()
      while true do
          for k,f in pairs(areas) do
              v = f
       player_in =  ( #(v.pos - _PLAYERCOORDS) <= v.radius) 
              if v.player_in and not player_in then -- was in: leave
          if v.leave == nil then 
            vRPserver.leaveArea({k})
          else
            v.leave()
          end
              playerInside = false
              elseif not v.player_in and player_in then -- wasn't in: enter
          if v.ent == nil then 
          insideName = v.name
          if(string.match(insideName,'vRP:gunshop')) then 
            insideName = 'gunshopul'
          end
          if(string.match(insideName,'vRP:garage')) then 
            insideName = 'garajul'
          end
          if(string.match(insideName,'hall')) then 
            insideName = 'Meniu Buletin'
          end
          if(string.match(insideName,'vRP:market')) then 
            insideName = 'magazinul'
          end
          if(string.match(insideName,'cloakroom')) then 
            insideName = 'vestiarul'
          end
          if(string.match(insideName,'vRP:mission')) then 
            insideName = 'misiunea'
          end
          if(string.match(insideName,'police:pc')) then 
            insideName = 'calculator politiei'
          end
          if(string.match(insideName,'static_')) then 
            insideName = 'meniul'
          end
          if(string.match(insideName,'vRP:gselector:Drug Dealer')) then 
            insideName = 'job Selector'
          end
          if(string.match(insideName,'vRP:seedSeller')) then 
            insideName = 'seller seminte'
          end
          if(string.match(insideName,'chest')) then 
            insideName = 'chestul'
          end
          playerInside = true  
          areaInside = k
        else
          v.ent()
              end
      end
      v.player_in = player_in -- update area player_in
          end
  
      Wait(300)
      end
  end)